function [class]=classifiers(Xtest,Xtrain,ytrain,classif)
% classifiers - a function for data classification; currently contains
% diagonal LDA and linear SVM classifiers but can be easily extended
%
% [class]=classifiers(Xtest,Xtrain,ytrain,classif)
%
% Input:
% Xtest... testing data matrix or vector (row(s) correspond to persons, 
%          columns to features)
% Xtrain.. training data matrix (rows correspond to persons, columns to
%          features)
% ytrain.. row vector with group identifiers of training data
% classif. vector of classifiers which are used (1..diagonal LDA, 2..linear
%          SVM); for example: [1,2] - both classifiers are used; 
%          [2] - linear SVM is used for classification)
%
% Output:
% class .. matrix of votes (group identifiers) which are results of
%          classification algorithms for each person (rows are classifiers,
%          columns are persons)
%
% by Eva Janousova, 29/MAR/2014
%
% ----------------------------------------------------------------------- %

class=zeros(size(Xtest,1),length(classif)); % allocation of matrix or vector with group identifiers of testing image(s)

for j=1:length(classif)
    switch(classif(j))
        case 1 % diagonal LDA:
            [class(:,j),~,~,~,~]=classify(Xtest,Xtrain,ytrain,'diaglinear');
        case 2 % linear SVM:
            svmStruct = svmtrain(Xtrain,ytrain);
            class(:,j) = svmclassify(svmStruct,Xtest);
        otherwise
            error('The parameter "classif" contains an unvalid identifier. Use "help classif_TtestFDRcor_LdaSvmRf_CorIncorLOO" for more information.')
    end
end

end

