function [effic]=cperformance(class,gis,pat)
% cperformance - a function for estimation of classification performance
%
% [effic]=cperformance(class,gis,pat)
%
% Input:
% class .. matrix of votes (group identifiers) which are results of
%          classification algorithms for each person (rows are
%          classifiers, columns are persons)
% gis .... vector with ground truth (true group identifiers)
% pat .... identifier of patients
%
% Output:
% effic .. efficiency of classifiers (i.e. classification performance)
%          (specifically: TP FN TN FP accur sens spec precis)
%
% by Eva Janousova, 9/SEP/2010 (modified 3/MAR/2014).

%------------------------------------------------------------------------%

i_pat=(gis==pat); % indexes of patient data
effic=zeros(size(class,1),8); % allocation of a matrix with performance measures

% computing efficiency of classifiers:
for i=1:size(class,1)
    eq =(class(i,:)==gis(:)'); % a logic vector: 0..classification is incorrect, 1..classification is correct

    TP=sum(eq(i_pat)); % true positive
    FN=sum(i_pat)-TP; % false negative
    TN=sum(eq(~i_pat)); % true negative
    FP=sum(~i_pat)-TN; % false positive

    accur = (TP+TN) / (TP+TN+FP+FN); % accuracy
    sens = TP / (TP+FN); % sensitivity
    spec = TN / (TN+FP); % specificity
    precis = TP / (TP+FP); % precision

    effic(i,:)=[TP FN TN FP accur sens spec precis]; % efficiency results
end

end