% demo_classif - an example of data reduction and classification of 2-D 
% image data of triangles and circles
%
% by Eva Janousova, 08/JAN/2015.

clear all
close all
echo on
clc

%------------------------------------------------------------------------
%         RECOGNITION OF 2-D IMAGE DATA OF TRIANGLES AND CIRCLES
%------------------------------------------------------------------------
 
%------------------------------------------------------------------------
% Data loading:
%------------------------------------------------------------------------

echo off
cpath=pwd;                              % current working directory
load([cpath,'\example_data.mat'])       % load the data
display('The data were loaded successfully.')
echo on


%------------------------------------------------------------------------
% Visualization of an example image:
%------------------------------------------------------------------------
echo off
i=0;
while (i<1 || i>20)
    i=input('Enter a number from 1 to 20 of an image which you want to visualize: ','s');
    i=str2num(i);
    if isempty(i)
        i=0;
    end
end
imagesc(reshape(X(i,:),246,204)),colormap('gray') % visualize the image

display(' ')
display('%------------------------------------------------------------------------')
display('% Image data reduction and classification:')

method=-1;
while (method~=0)
    echo on
%------------------------------------------------------------------------
    
% Select data reduction method and type of leave-one-out cross-validation:
% 0 ... end the demo
% 1 ... pLDA with resampling with correct LOOCV
% 2 ... pLDA with resampling with incorrect LOOCV
% 3 ... pLR with resampling with correct LOOCV
% 4 ... pLR with resampling with incorrect LOOCV
% 5 ... pPCA with correct LOOCV
% 6 ... pPCA with incorrect LOOCV
% 7 ... t-test selecting features with p-value < alpha with correct LOOCV
% 8 ... t-test selecting features with p-value < alpha with incorrect LOOCV
% 9 ... t-test selecting N most disciminative features with correct LOOCV
% 10 .. t-test selecting N most disciminative features with incorrect LOOCV
echo off
    method=input('Enter a number of the chosen combination: ','s');
    method=str2num(method);

    if isempty(method)
        method=-1;
    end
    
    switch(method)

case 0
    break

case 1
    echo on
            
    % Setting parameters:
    group=1;        % identifier of triangles
    classif=[1,2];  % classifiers (1..diagonal LDA, 2..linear SVM)
    loo=0;          % type of LOOCV (0..correct LOOCV, 1..incorrect LOOCV)
    thresh=1e-07;   % threshold to determine when the algorithm converges
    lambda=0.3;     % regularization parameter in pLDA 
    prob=0.99;      % probability threshold for selection of features
    B=10;           % number of iterations of the resampling
    
    % Recognition:
    [effic,class,gis,nit,fsel,nsel]=classif_pLDA_loo(X,y,group,classif,loo,thresh,lambda,prob,B);
    
    % Comparison of classification performance:
    [p]=mcnemar(class',gis);
    
    % Visuzalization of the results in a figure
    
    echo off
    figname='pLDA with resampling with correct LOOCV';
    
            
case 2
    echo on
            
    % Setting parameters:
    group=1;        % identifier of triangles
    classif=[1,2];  % classifiers (1..diagonal LDA, 2..linear SVM)
    loo=1;          % type of LOOCV (0..correct LOOCV, 1..incorrect LOOCV)
    thresh=1e-07;   % threshold to determine when the algorithm converges
    lambda=0.3;     % regularization parameter in pLDA 
    prob=0.99;      % probability threshold for selection of features
    B=10;           % number of iterations of the resampling
    
    % Recognition:
    [effic,class,gis,nit,fsel,nsel]=classif_pLDA_loo(X,y,group,classif,loo,thresh,lambda,prob,B);
    
    % Comparison of classification performance:
    [p]=mcnemar(class',gis);
    
    % Visuzalization of the results in a figure
    
    echo off
    figname='pLDA with resampling with incorrect LOOCV';
    

case 3
    echo on
            
    % Setting parameters:
    group=1;        % identifier of triangles
    classif=[1,2];  % classifiers (1..diagonal LDA, 2..linear SVM)
    loo=0;          % type of LOOCV (0..correct LOOCV, 1..incorrect LOOCV)
    lambda=0.6;     % regularization parameter in pLDA 
    prob=0.5;       % probability threshold for selection of features
    B=10;           % number of iterations of the resampling
    
    % Recognition:
    [effic,class,gis,fsel,nsel]=classif_pLR_loo(X,y,group,classif,loo,lambda,prob,B);
    
    % Comparison of classification performance:
    [p]=mcnemar(class',gis);

    % Visuzalization of the results in a figure
    
    echo off
    figname='pLR with resampling with correct LOOCV';
    
    
case 4
    echo on
            
    % Setting parameters:
    group=1;        % identifier of triangles
    classif=[1,2];  % classifiers (1..diagonal LDA, 2..linear SVM)
    loo=1;          % type of LOOCV (0..correct LOOCV, 1..incorrect LOOCV)
    lambda=0.6;     % regularization parameter in pLDA 
    prob=0.5;       % probability threshold for selection of features
    B=10;           % number of iterations of the resampling
    
    % Recognition:
    [effic,class,gis,fsel,nsel]=classif_pLR_loo(X,y,group,classif,loo,lambda,prob,B);
    
    % Comparison of classification performance:
    [p]=mcnemar(class',gis);

    % Visuzalization of the results in a figure
    
    echo off
    figname='pLR with resampling with incorrect LOOCV';
    
    
case 5
    echo on
            
    % Setting parameters:
    group=1;        % identifier of triangles
    classif=[1,2];  % classifiers (1..diagonal LDA, 2..linear SVM)
    loo=0;          % type of LOOCV (0..correct LOOCV, 1..incorrect LOOCV)
    m=15;           % 15 eigenvectors explaning less variablity will be discarded
    
    % Recognition:
    [effic,class,gis]=classif_isPCA_loo(X,y,group,classif,loo,m);
    
    % Comparison of classification performance:
    [p]=mcnemar(class',gis);

    % Visuzalization of the results in a figure
    
    echo off
    figname='PCA with correct LOOCV';
    
    
case 6
    echo on
            
    % Setting parameters:
    group=1;        % identifier of triangles
    classif=[1,2];  % classifiers (1..diagonal LDA, 2..linear SVM)
    loo=1;          % type of LOOCV (0..correct LOOCV, 1..incorrect LOOCV)
    m=15;           % 15 eigenvectors explaning less variablity will be discarded
    
    % Recognition:
    [effic,class,gis]=classif_isPCA_loo(X,y,group,classif,loo,m);
    
    % Comparison of classification performance:
    [p]=mcnemar(class',gis);

    % Visuzalization of the results in a figure
    
    echo off
    figname='PCA with incorrect LOOCV';
    
    
case 7
    echo on
            
    % Setting parameters:
    group=1;        % identifier of triangles
    classif=[1,2];  % classifiers (1..diagonal LDA, 2..linear SVM)
    alpha=0.05;     % threshold for p-values calculated by t-test
    loo=0;          % type of LOOCV (0..correct LOOCV, 1..incorrect LOOCV)
    
    % Recognition:
    [effic,class,gis,fsel]=classif_Ttest_loo(X,y,group,classif,alpha,loo);
    
    % Comparison of classification performance:
    [p]=mcnemar(class',gis);

    % Visuzalization of the results in a figure
    
    echo off
    figname='t-test p<alpha with correct LOOCV';

    
case 8
    echo on
            
    % Setting parameters:
    group=1;        % identifier of triangles
    classif=[1,2];  % classifiers (1..diagonal LDA, 2..linear SVM)
    alpha=0.05;     % threshold for p-values calculated by t-test
    loo=1;          % type of LOOCV (0..correct LOOCV, 1..incorrect LOOCV)
    
    % Recognition:
    [effic,class,gis,fsel]=classif_Ttest_loo(X,y,group,classif,alpha,loo);
    
    % Comparison of classification performance:
    [p]=mcnemar(class',gis);
        
    % Visuzalization of the results in a figure
    
    echo off
    figname='t-test p<alpha with incorrect LOOCV';
    
        
case 9
    echo on
            
    % Setting parameters:
    group=1;        % identifier of triangles
    classif=[1,2];  % classifiers (1..diagonal LDA, 2..linear SVM)
    nsel=1000;      % number of most significant features ordered based on t-test
    loo=0;          % type of LOOCV (0..correct LOOCV, 1..incorrect LOOCV)
    
    % Recognition:
    [effic,class,gis,fsel]=classif_Ttest_nsel_loo(X,y,group,classif,nsel,loo);
    
    % Comparison of classification performance:
    [p]=mcnemar(class',gis);
    
    % Visuzalization of the results in a figure
    
    echo off
    figname='t-test with correct LOOCV';
   
            
case 10
    echo on
            
    % Setting parameters:
    group=1;        % identifier of triangles
	classif=[1,2];  % classifiers (1..diagonal LDA, 2..linear SVM)
    nsel=1000;      % number of most significant features ordered based on t-test
    loo=1;          % type of LOOCV (0..correct LOOCV, 1..incorrect LOOCV)
            
    % Recognition:
    [effic,class,gis,fsel]=classif_Ttest_nsel_loo(X,y,group,classif,nsel,loo);
    
    % Comparison of classification performance:
    [p]=mcnemar(class',gis);
    
    % Visuzalization of the results in a figure
            
    echo off
    figname='t-test with incorrect LOOCV';

otherwise
    method=-1;
    
    end
    
    % Visuzalization:
    if method~=-1
        figure('name',figname)
        subplot(1,2,1), bar([effic(1,(size(effic,2)-3):(size(effic,2)-1));effic(2,(size(effic,2)-3):(size(effic,2)-1))],'grouped'); % visualization of classification performance
        title(['Classification performance (p=',num2str(p,'%0.3f'),')'])
        axis([0.5,2.5,0,1.35])
        colormap([0 1 1; 0 0 1; 0 0 0.7])
        text([0.7,0.925,1.15,1.7,1.925,2.15]',0.04+[effic(1,(size(effic,2)-3):(size(effic,2)-1)),effic(2,(size(effic,2)-3):(size(effic,2)-1))]',num2str([effic(1,(size(effic,2)-3):(size(effic,2)-1)),effic(2,(size(effic,2)-3):(size(effic,2)-1))]','%0.2f'),'FontSize',6)
        legend({'accuracy','sensitivity','specificity'},'Location','northeast','FontSize',7)
        set(gca,'XTickLabel',{'LDA','linSVM'})
        set(gca,'FontSize',8)
        if (method~=5 && method~=6)
            subplot(1,2,2),hold off,imagesc(reshape(fsel,246,204)) % visualization of the selected features
            title('Selected features')
            set(gca,'XTick',[],'YTick',[])
        end
        colormap('gray')
    end
    
end

clear all
