function [Xred,pvar,Xmean,V]=isPCA_reduc(X,m)
% isPCA_reduc - a function for reduction of 2-D data matrix using   
% intersubject principal component analysis (isPCA); in matrix X a number  
% of subjects(rows) is much smaller than number of features (columns).
% 
% [Xred,Xmean,V,pvar]=isPCA_reduc(X,m)
%
% Input:
% X ...... data matrix (rows correspond to persons, columns to features)
% m ...... number of eigenvectors which you want to discard (a possitive 
%          number means discarding N eigenvectors with smallest 
%          eigenvalues, a negative number means discarding N eigenvectors 
%          with largest eigenvalues and zero means discarding no 
%          eigenvectors); absolute value of m must be less than a number of
%          images minus one
%
% Output:
% Xred ... reduced data matrix by isPCA
% pvar ... explained variability by isPCA
% Xmean .. mean person image (mean over all features)
% V ...... isPCA projection matrix which consists of eigenvectors
%
% by Eva Janousova, 27/AUG/2010.

%------------------------------------------------------------------------%

% subtract mean person image from matrix X:
Xmean=mean(X); % mean person image
Xmean_mult=ones(size(X,1),1)*Xmean; % multiplicate the mean person image (Xmean_mult is a matrix where rows are the mean person image)
X=X-Xmean_mult; % subtract the mean person image matrix from the matrix X
clear Xmean_mult
    
% reduce centered image matrix X using isPCA projection matrix V:
[V,pvar] = isPCA_eig(X,m); % compute projection matrix V
Xred=X*V; % reduce mean-centered matrix X

end
