function [p]=mcnemar(class2,gis)
% mcnemar - a function for calculating p-value using McNemar's test for 
% comparison of two classifiers (based on the formula 1.11 from Kuncheva,   
% L.I., 2004. Combining Classifiers: Ideas and Methods. John Wiley & Sons,
% Hoboken, NJ.)
%
% [p]=mcnemar(class2,gis)
%
% Input:
% class2 .. matrix of votes (group identifiers) which are results of
%          2 classification algorithms for each person (rows are
%          classifiers, columns are persons)
% gis .... vector with ground truth (true group identifiers)
%
% Output:
% p ...... p-value calculated using McNemar's test for comparison of 2
%          classifiers
%
% by Eva Janousova, 15/APR/2014
%
% ---------------------------------------------------------------------- %

if size(class2,1)~=2
    error('The matrix "class2" must have 2 rows.')
end

if size(class2,2)~=length(gis)
    error('The length of "gis" and the number of columns in "class2" must be equal.')
end

eq=(class2==(ones(2,1)*gis(:)')); % a logic matrix: 0..classification is incorrect, 1..classification is correct
N01=sum((eq(1,:)==0).*(eq(2,:)==1)); % calculating N01
N10=sum((eq(1,:)==1).*(eq(2,:)==0)); % calculating N10

if N01+N10~=0
    chi2=((abs(N01-N10)-1)^2)/(N01+N10); % test statistics (according to the formula 1.11)
    p=1-chi2cdf(chi2,1); % p-value
else
    p=1; % p-value
end

end





