function [v,niters]=pLDA(m1,m2,s,lambda,thresh)
% pLDA - a function for computation of penalised linear discriminant 
% analysis
%
% [v,niters]=pLDA(m1,m2,s,lambda,thresh)
%
% Input:
% m1 ..... mean vector of group 1
% m2 ..... mean vector of group 2
% s ...... within variance vector
% lambda . regularization parameter
% thresh . threshold to be used as an assessment of convergence
%
% Output:
% v ...... regression coefficients of pLDA
% niters . number of iterations until convergence of pLDA 
%
% by Eva Janousova, 22/MAY/2011.

%------------------------------------------------------------------------%

% identification and removal of features with nonzero variance
nonzero=(s>eps); 
s=s(nonzero);
m1=m1(nonzero);
m2=m2(nonzero);

v0=((m1-m2)/sqrt(sum((m1-m2).^2)))./s; % initialize v0 and normalize the vector of difference between the means

niters=0;
convflag=false;

while(not(convflag))
    niters = niters+1;
    a=((m1-m2)*((m1-m2)'*v0))./s;
    
    % soft thresholding in terms of regularization parameter lambda
    S=abs(a)-lambda; 
    S(S<=0)=0;
    S_v=sign(a).*S;

    v1=(S_v/sqrt(sum(S_v.^2)))./s;
    
    % checking the convergence
    conv_criterion=(abs(1-(abs(sum(v1.*v0.*(s.^2)))))<thresh);
    if (conv_criterion || niters>=1000) 
        convflag=true;
    end
    if (not(convflag)) 
        v0=v1;
    end
    
end

% transformation of the vector with regression coefficients to have the same length as original vectors
v=zeros(length(nonzero),1);
v(nonzero)=v1;

end
    



