function [A]=pLR(X,y,lambda)
% pLR - a function for computation of penalised linear regression (pLR)
%
% [A]=pLR(X,y,lambda)
%
% Input:
% X ...... data matrix (rows correspond to persons, columns to features)
% y ...... row vector with group identifiers
% lambda . regularization parameter
%
% Output:
% A ...... matrix with 0 and 1 (1..feature was selected); number of rows of 
%          A is equal to length(lambda) and number of columns is equal to  
%          number of features
%
% by Eva Janousova, 22/FEB/2011.

%------------------------------------------------------------------------%

% number of subjects:
n = size(X,1); 

% mean-centering of y:
y = y - mean(y); % mean-centering of y

% mean-centering and scaling of X:
meanx = mean(X); % mean image
for j=1:n 
    X(j,:)=X(j,:) - meanx; % mean-centering of X (it could be done using matrix subtraction instead of cycle however it would have very high memory requirements)
end
normx = sqrt(ones(1,n)*(X.^2)/(n-1)); % normalization of X
inx_varnon0=(normx>eps); % indexes of features with nonzero variance
X=X(:,inx_varnon0); % selection of features with nonzero variance
normx=normx(inx_varnon0);
X=X./(ones(size(X,1),1)*normx);

% pLR:
Z = X' * y;
Z = Z/sqrt(sum(Z.^2)); % normalisation
A1 = zeros(length(lambda),size(X,2)); % allocation of a matrix A1
for i = 1:length(lambda)
    valb = abs(Z) - lambda(i) * max(abs(Z)); % soft thresholding
    A1(i,:) = (valb>=0); % selection of features with non-zero coefficients
end

% transformation of A to have the same number of columns as the matrix X:
A=zeros(size(A1,1),length(inx_varnon0));
A(:,inx_varnon0)=A1;

end
