function [perc]=stability_pLR(X,y,B,lambda)
% stability_pLR - a function for feature selection using penalised linear 
% regression (pLR) with resampling
%
% [perc]=stability_pLR(X,y,B,lambda)
%
% Input: 
% X ...... data matrix (rows correspond to persons, columns to features)
% y ...... row vector with group identifiers
% B ...... number of iterations in resampling
% lambda . regularization parameter
%
% Output:
% perc ... percentages of how often feature was selected to play role in
%          pLR model
%
% NOTE: 
% The function is now designed for analysis of 2 groups of subjects only! 
%
% by Eva Janousova, 24/FEB/2011.

%------------------------------------------------------------------------%

% initial settings:
[~,p] = size(X); % p..number of predictors (features)
if (size(y,2)>size(y,1))
    y=y'; % y must be a column vector
end
groups = tabulate(y); % a frequency table of group identifiers for detection of a number of groups of images
groups = groups((groups(:,2)>0),1); % creating a vector with values of identifiers
inx1 = find(y==groups(1)); % indexes of subjects from the 1st group
inx2 = find(y==groups(2)); % indexes of subjects from the 2nd group
perc = zeros(length(lambda),p); % allocation of a matrix with percentages of how many times the feature was selected

% checking values of the parameter lambda:
if (min(lambda)<0 || max(lambda)>=1)
    error('Lambda should be non-negative and less than 1 !')
end

% recode y into a vector with 0 and 1:
y_code = zeros(size(y));
y_code(y==groups(2))=1;

% stability selection + PLR:
for b=1:B % number of iterations of stability selection
        inxr1 = randperm(length(inx1)); % random permutation of numbers from 1 to length(inx1)
        inxr2 = randperm(length(inx2)); % random permutation of numbers from 1 to length(inx2)
        inx_half = [inx1(inxr1(1:floor(length(inx1)/2)));inx2(inxr2(1:floor(length(inx2)/2)))]; % random selection of half of subjects from the 1st group and half of subjects from the 2nd group
        perc=perc+pLR(X(inx_half,:),y_code(inx_half),lambda); % pLR      
end
perc=perc/B; % calculating the percentages of feature selections

end



